@@ -23,12 +23,16 @@ script. Note that this requires you to have zxpy installed globally.
 """
 import ast
 import code
+import codecs
+import contextlib
 import inspect
-import subprocess
 import shlex
+import subprocess
 import sys
 import traceback
-from typing import Tuple, Union, IO
+from typing import Generator, Tuple, Union, IO
+
+UTF8Decoder = codecs.getincrementaldecoder("utf8")
 
 
 def cli() -> None:
@@ -56,38 +60,41 @@ def cli() -> None:
         run_zxpy(filename, module)
 
 
-def create_shell_process(command: str) -> IO[bytes]:
-    """Creates a shell process, returning its stdout to read data from."""
+@contextlib.contextmanager
+def create_shell_process(command: str) -> Generator[IO[bytes], None, None]:
+    """Creates a shell process, yielding its stdout to read data from."""
     process = subprocess.Popen(
         command,
         stdout=subprocess.PIPE,
         stderr=subprocess.STDOUT,
         shell=True,
     )
+    assert process.stdout is not None
+    yield process.stdout
+
     process.wait()
+    process.stdout.close()
     if process.returncode != 0:
         raise ChildProcessError(process.returncode)
 
-    assert process.stdout is not None
-    return process.stdout
-
 
 def run_shell(command: str) -> str:
     """This is indirectly run when doing ~'...'"""
-    stdout = create_shell_process(command)
-    return stdout.read().decode()
+    with create_shell_process(command) as stdout:
+        output = stdout.read().decode()
+        return output
 
 
 def run_shell_print(command: str) -> None:
     """Version of `run_shell` that prints out the response instead of returning a string."""
-    stdout = create_shell_process(command)
-    while True:
-        char = stdout.read(1)
-        if not char:
-            break
+    with create_shell_process(command) as stdout:
+        decoder = UTF8Decoder()
+        with open(stdout.fileno(), 'rb', closefd=False) as buff:
+            # read1 should be present on BinaryIO but it's not. Typeshed bug
+            for text in iter(buff.read1, b""):  # type: ignore
+                print(decoder.decode(text), end="")
 
-        sys.stdout.buffer.write(char)
-        sys.stdout.flush()
+            print(decoder.decode(b"", final=True), end="")
 
 
 def run_shell_alternate(command: str) -> Tuple[str, str, int]:
@@ -98,14 +105,15 @@ def run_shell_alternate(command: str) -> Tuple[str, str, int]:
         stderr=subprocess.PIPE,
         shell=True,
     )
-    process.wait()
+
+    stdout_text, stderr_text = process.communicate()
     assert process.stdout is not None
     assert process.stderr is not None
     assert process.returncode is not None
 
     return (
-        process.stdout.read().decode(),
-        process.stderr.read().decode(),
+        stdout_text.decode(),
+        stderr_text.decode(),
         process.returncode,
     )
 
@@ -271,6 +279,9 @@ def install() -> None:
             new_input = input(prompt)
         except KeyboardInterrupt:
             print()
+            if continued_command:
+                continued_command = False
+                command = ""
             continue
         except EOFError:
             print()
